(*
 * <COPYRIGHT>
 * Copyright 2005 Gerd Stolpmann
 *
 * <GPL>
 * This file is part of UMLMON.
 *
 * UMLMON is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * UMLMON is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with UMLMON; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * </>
 *)

(* $Id: mount.mli 187 2005-10-14 10:54:15Z gerd $ *)

exception Already_mounted
exception Not_a_mount_point
exception Mount_point_not_empty
exception Mount_error

val is_mount_point : string -> bool
  (* Checks whether the passed directory or file is a mount point. The
   * check bases on /proc/mounts.
   *)

val mount_bind_dir : string -> string -> unit
  (* [mount_bind_dir target link]: Ensures that:
   * - [link] is a directory owned by root
   * - [link] is a mount point doing a bind mount to [target]
   *
   * If [link] is already a mount point, the function will raise
   * [Already_mounted], even if the desired mount is already
   * established (it is not possible to check that).
   *
   * If [link] is not a mount point, it is checked whether [link]
   * is a an existing directory. If so, it is required that it is
   * empty, or [Mount_point_not_empty] is raised. If not, it is
   * created. Independent of this, root becomes owner of the
   * directory with permissions 0o000.
   *
   * Raises [Mount_error] if the invoked mount command indicates an error.
   *)

val mount_bind_file : string -> string -> unit
  (* [mount_bind_file target link]: Ensures that:
   * - [link] is a file owned by root
   * - [link] is a mount point doing a bind mount to [target]
   *
   * If [link] is already a mount point, the function will raise
   * [Already_mounted], even if the desired mount is already
   * established (it is not possible to check that).
   *
   * If [link] is not a mount point, it is checked whether [link]
   * is a an existing regular file. If so, it is required that it is
   * empty, or [Mount_point_not_empty] is raised. If not, it is
   * created. Independent of this, root becomes owner of the
   * file with permissions 0o000.
   *
   * Raises [Mount_error] if the invoked mount command indicates an error.
   *)

val mount_tmpfs : int -> string -> unit
  (* [mount_tmpfs size link]: Ensures that:
   * - [link] is a directory owned by root
   * - A tmpfs filesystem is mounted at [link] with [size] megs
   *
   * If [link] is already a mount point, the function will raise
   * [Already_mounted], even if the desired mount is already
   * established (it is not possible to check that).
   *
   * If [link] is not a mount point, it is checked whether [link]
   * is a an existing directory. If so, it is required that it is
   * empty, or [Mount_point_not_empty] is raised. If not, it is
   * created. Independent of this, root becomes owner of the
   * directory with permissions 0o755.
   *
   * Raises [Mount_error] if the invoked mount command indicates an error.
   *)

val umount : string -> unit
  (* [umount link]: If [link] is a mount point, it is unmounted.
   * Otherwise [Not_a_mount_point] is raised.
   *
   * Raises [Mount_error] if the invoked mount command indicates an error.
   *)
