/* -*- c -*- 
 * <COPYRIGHT>
 * Copyright 2005 Gerd Stolpmann
 *
 * <GPL>
 * This file is part of UMLMON.
 *
 * UMLMON is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * UMLMON is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with UMLMON; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * </>
 */

/* $Id: umlmonctrl.x 209 2005-10-23 13:46:46Z gerd $ */

/* INTERFACE DEFINITION FILE FOR THE UMLMON PROTOCOLS
 *
 * This file contains the IDL for the RPC protocol UMLMON uses to send commands to
 * UMLMON monitors. This is basically ONC RPC (also known as Sun RPC), as defined in
 * RFC 1831 and RFC 1832. Note, however, the following special arrangements:
 *
 * - UMLMON uses Unix Domain Sockets in stream mode by default. This is not part of
 *   the mentioned RFCs (which only know RPC over IP). The protocol used for the Unix
 *   Domain Sockets is the same that the RFCs define for TCP.
 *
 * - Optionally, UMLMON also binds to a real TCP port.
 *
 * - UMLMON does not use the portmapper. Instead, UMLMON has its own directory service.
 *   This service is contained in this file as program "Directory" (see below for 
 *   details).
 *
 * The following programs are defined in this file:
 *
 * - Ctrl: This is the control protocol of the monitors. Every monitor opens such
 *   a socket (as /var/lib/umlmon/<name>/ctrl, and optionally as TCP port).
 *
 * - Directory: The directory service. The socket is /var/lib/umlmon/directory/directory,
 *   and optionally there is a TCP port, too (the port called "umldir" in /etc/services).
 *
 * - Internal: The protocol used between the control process and the monitor process.
 *   There are no public ports.
 *
 * On RPC level all connections are unauthenticated (AUTH_NONE). Connections to
 * Ctrl with TCP must be authenticated by calling the authentication procedures
 * before calling any other procedure.
 *
 * Hint: This is "new style RPC", so you need to call rpcgen with option -N.
 *
 * Versions: The version numbers of the RPC protocol represent "major version numbers".
 * There is a core of procedures defined for every major version. There will
 * also be minor versions, and these versions can:
 * - add new procedures in the extension area (see below)
 * - add new parameters where the protocol explicitly allows this
 * It is expected that clients and servers of the same major version can communicate
 * with each other, but that some functions may be unavailable if the minor versions
 * do not match.
 *
 * MAJOR  MINOR   SVN         COMMENT
 * 1      0       186         First version released as Open Source, serving as
 *                            reference
 * 1      1       ???         Added sync_disks extension (UMLMON 1.0.2)
 */

const minor_ctrl_version = 1;
/* Minor version of the Ctrl program */

typedef string longstring<>;
/* An arbitrary long string */

typedef longstring *longstring_opt;
/* An optional long string */

typedef string sysrq_arg<1>;
/* A string of length 1, used as argument for the sysrq procedure */

typedef int *int_opt;
/* An optional integer */

enum uml_status_code {
    CTRL_INACTIVE = 0,    /* The machine is inactive */
    CTRL_STOPPED = 1,     /* The machine is suspended */
    CTRL_RUNNING = 2,     /* The machine is running */
    CTRL_STARTING = 3     /* The machine is currently starting */
};
/* The status code reporting the current status of the machine */

union uml_status switch (uml_status_code c) {
 case CTRL_INACTIVE: void;
 case CTRL_STOPPED: void;
 case CTRL_RUNNING: void;
 case CTRL_STARTING: void;
};
/* A "union" of all statuses, for sane typing */


enum term_kind {
    CTRL_PTY = 0
};
/* The ways terminals (consoles and serial lines) can be monitored. There is currently
 * only one way, CTRL_PTY (corresponding to "pty" in the configuration files).
 */


struct pty {
    longstring    *ctrl_ptsfile;   /* NULL if not active */
    longstring    *ctrl_logfile;   /* NULL if not available */
    bool          ctrl_connected;
};
/* Describes the CTRL_PTY monitoring method in detail:
 * - ctrl_ptsfile is the path of the (slave) terminal device
 * - ctrl_logfile is the optional path to the logfile
 * - ctrl_connected is true when a user is currently connected to the terminal
 *   (e.g. using "umlwatch connect")
 */


union term_def switch (term_kind t) {
 case CTRL_PTY:
     pty pty;
};
/* The monitoring method of the terminal */


struct uml_mon_term {
    longstring    ctrl_termname;
    term_def      ctrl_termdef;
    longstring    ctrl_extension<>;
};
/* The terminal and the monitoring method. ctrl_termname is the name such as "con3"
 * or "ssl0".
 *
 * ctrl_extension: An array of tokens with enabled extensions.
 * List of defined extensions:
 * MAJOR  MINOR   EXTENSION    COMMENT
 * -------------------------------------------------------------------------------
 * 1      0       -            no extension defined
 */


typedef uml_mon_term uml_mon_terms<>;
/* All monitored terminals */


typedef longstring arg_list<>;
/* List of string arguments */


enum code {
    CODE_OK = 0,
    CODE_ERROR = 1,
    CODE_INTERRUPT = 2,
    CODE_IN_PROGRESS = 3
};
/* General return codes of state-changing procedures:
 * - CODE_OK: The state change was successful
 * - CODE_ERROR: The state change was not successful
 * - CODE_INTERRUPT: The state change has been interrupted in the middle. This return
 *   code is only used for certain procedures that block until the state change can
 *   be carried out, but the procedures are interrupted while waiting because of some
 *   reason.
 * - CODE_IN_PROGRESS: The state change is done in the background.
 */


union r_simple switch (code c) {
 case CODE_OK:
     void;
 case CODE_ERROR:
     longstring error_message;
 case CODE_INTERRUPT:
     void;
 case CODE_IN_PROGRESS:
     void;
};
/* Return value for many simple procedures. */


struct r_logfile_read_success {
    int line_start;      /* line number of the first line */
    longstring lines<>;  /* The lines */
};
/* When a log file is read, this struct contains the read part of the file:
 * - line_start: is the line number of the first returned line (starting with 1)
 * - lines: are the lines of the file
 */


union r_logfile_read switch (code c) {
 case CODE_OK:
     r_logfile_read_success success;
 case CODE_ERROR:
     longstring error_message;
 case CODE_INTERRUPT:     /* Currently not possible */
     void;
 case CODE_IN_PROGRESS:   /* Currently not possible */
     void;
};
/* Return value for the "logfile_read" procedure. On success, the read part of the
 * log file is returned
 */


struct term_handle {
    longstring  ct_name;   /* ct = connected terminal */
    int         ct_id;
};
/* Identifies a connected terminal by:
 * - ct_name: is the name of the terminal such as "con4" or "ssl0"
 * - ct_id: is the connection ID as returned by the "connect" procedure
 */


union r_connect switch (code c) {
 case CODE_OK:
     term_handle term_handle;
 case CODE_ERROR:
     longstring error_message;
 case CODE_INTERRUPT:     /* Currently not possible */
     void;
 case CODE_IN_PROGRESS:   /* Currently not possible */
     void;
};
/* Return value of the "connect" procedure. On success a term_handle is returned that
 * can be used to identify the connected terminal in further procedure calls.
 */


union r_read switch (code c) {
 case CODE_OK:
     longstring rd_data;
 case CODE_ERROR:
     longstring error_message;
 case CODE_INTERRUPT:
     void;
 case CODE_IN_PROGRESS:   /* Currently not possible */
     void;
};
/* Return value of the "read" procedure. On success the read data are returned in a
 * longstring. Note that CODE_INTERRUPT is possible for "read".
 */


union r_write switch (code c) {
 case CODE_OK:
     void;
 case CODE_ERROR:
     longstring error_message;
 case CODE_INTERRUPT:
     int wr_length;
 case CODE_IN_PROGRESS:    /* Currently not possible */
     void;
};
/* Return value of the "write" procedure. Note that CODE_INTERRUPT is possible for
 * "write". In wr_length the number of actually written bytes are returned until
 * the interruption happened.
 */


union r_disconnect switch (code c) {
 case CODE_OK:
     void;
 case CODE_ERROR:
     longstring error_message;
 case CODE_INTERRUPT:     /* Currently not possible */
     void;
 case CODE_IN_PROGRESS:   /* Currently not possible */
     void;
};
/* Return value of the "disconnect" procedure. */


enum key_type {
    KEY_TRANSLATED = 0,
    KEY_CURSOR_UP = 1,
    KEY_CURSOR_DOWN = 2,
    KEY_CURSOR_LEFT = 3,
    KEY_CURSOR_RIGHT = 4,
    KEY_PAGE_UP = 5,
    KEY_PAGE_DOWN = 6,
    KEY_HOME = 7,
    KEY_END = 8,
    KEY_INSERT = 9,
    KEY_DELETE = 10,
    KEY_BACKSPACE = 11,
    KEY_ENTER = 12,
    KEY_TAB = 13,
    KEY_ESCAPE = 14,
    KEY_FN = 15,
    KEY_NAMED = 99
};
/* The various key types, see "union key" below for explanations */


union key switch (key_type t) {
 case KEY_TRANSLATED:
     int key_unicode;
 case KEY_CURSOR_UP: void;
 case KEY_CURSOR_DOWN: void;
 case KEY_CURSOR_LEFT: void; 
 case KEY_CURSOR_RIGHT: void;
 case KEY_PAGE_UP: void;
 case KEY_PAGE_DOWN: void;
 case KEY_HOME: void;
 case KEY_END: void;
 case KEY_INSERT: void;
 case KEY_DELETE: void;
 case KEY_BACKSPACE: void;
 case KEY_ENTER: void;
 case KEY_TAB: void;
 case KEY_ESCAPE: void;
 case KEY_FN:
     int key_number;  /* 1..12 */
 case KEY_NAMED:
     longstring key_name;
};
/* The supported keys:
 * - KEY_TRANSLATED means a key identified by its Unicode code. This includes all
 *   control keys, too.
 * - KEY_CURSOR_UP ... KEY_ESCAPE: Various special keys
 * - KEY_FN: Means a function key. The key_number can range from 1 to 12.
 * - KEY_NAMED: Select a control key by name. This should also be used for
 *   extensions, if added.
 *
 * Modified keys (with Ctrl, Alt, Meta, ...) are not supported.
 */


typedef key keys<>;
/* A sequence of key presses */


const MASK_INTENSITY  = 0x0003;
const MASK_UNDERSCORE = 0x0004;
const MASK_BLINK      = 0x0008;
const MASK_REVERSE    = 0x0010;
const MASK_FG_COLOR   = 0x0F00;
const MASK_BG_COLOR   = 0xF000;
/* Bitmasks of cell_atts (below) */


const SHIFT_INTENSITY  = 0;
const SHIFT_UNDERSCORE = 2;
const SHIFT_BLINK      = 3;
const SHIFT_REVERSE    = 4;
const SHIFT_FG_COLOR   = 8;
const SHIFT_BG_COLOR   = 12;
/* Corresponding shift offsets */


enum intensity {
    I_norm = 0,
    I_dim = 1,
    I_bold = 2
};
/* Intensity attribute of the Linux console */


enum underscore {
    U_off = 0,
    U_on = 1
};
/* Underscore attribute of the Linux console */


enum blink {
    B_off = 0,
    B_on = 1
};
/* Blink attribute of the Linux console */


enum reverse {
    R_off = 0,
    R_on = 1
};
/* Reverse video attribute of the Linux console */


enum color {
    C_BLACK = 0,
    C_RED = 1,
    C_GREEN = 2,
    C_BROWN = 3,
    C_BLUE = 4,
    C_MAGENTA = 5,
    C_CYAN = 6,
    C_WHITE = 7
};
/* Color attribute of the Linux console */


struct cell {
    int  cell_char;   /* -1: Empty, else a Unicode point */
    int  cell_atts;   /* Attributes */
};
/* A cell of the Linux console. In cell_char, the Unicode code of the displayed 
 * character is stored, or -1 for an empty cell. In cell_atts the attributes
 * are encoded as follows:
 *
 * - Every attribute (intensity, underscore, blink, reverse, foreground color,
 *   background color) is represented by several bits in cell_atts.
 *
 * - The bits are the one bits in the MASK_* constant. For example, reverse video
 *   is expressed by the single bit at binary position 4.
 *
 * - The number of the corresponding "enum" is written into the selected bits.
 *   For example, reverse video off is a 0 bit, and on is a 1 bit.
 *
 * - The SHIFT_* constants are helpers to ease programming.
 */


typedef cell row<>;
/* A row of cells */


struct r_te_state {
    int  te_height;
    int  te_width;
    int  te_cursor_x;
    int  te_cursor_y;
    bool te_cursor_visible;
    row  te_screen<>;
};
/* The state of the Linux console:
 * - te_height: The number of rows
 * - te_width: The number of columns
 * - te_cursor_x: The X position of the cursor (from 0 to te_width-1)
 * - te_cursor_y: The Y position of the cursor (from 0 to te_height-1)
 * - te_cursor_visible: Whether the cursor is visible
 * - te_screen: The contents of the screen as matrix of rows and columns.
 *   te_screen[0][0] is the upper left corner of the screen.
 */


typedef r_te_state *r_te_state_opt;
/* Optional Linux console state */


union r_site switch (code c) {
 case CODE_OK:
     longstring site_result;
 case CODE_ERROR:
     longstring error_message;
 case CODE_INTERRUPT:      /* Currently not possible */
     void;
 case CODE_IN_PROGRESS:    /* Currently not possible */
     void;
};
/* Return value of the site command: On success, site_result is the output of the
 * command.
 */


enum disk_status {
    DISK_EXISTS = 0,
    DISK_BEING_CREATED = 1,
    DISK_ERROR = 2
};
/* The status of a disk managed by UMLMON:
 * - DISK_EXISTS: The disk exists, and no special condition is known.
 * - DISK_BEING_CREATED: The disk is currently being created or being modified.
 *     The operation is still in progress.
 * - DISK_ERROR: There was an error for the last operation targeting this disk.
 */


enum lock_status {
    LOCK_NULL = 0,      /* No lock */
    LOCK_READ = 1,      /* Read lock */
    LOCK_READ_OWN = 2,  /* Read lock, also held by this VM */
    LOCK_WRITE = 3,     /* Write lock */
    LOCK_WRITE_OWN = 4  /* Write lock held by this VM */
};


union lock_info switch (lock_status ls) {
 case LOCK_NULL: void;
 case LOCK_READ: void;
 case LOCK_READ_OWN: void;
 case LOCK_WRITE: void;
 case LOCK_WRITE_OWN: void;
};


struct disk_info {
    hyper disk_size;           /* logical size in bytes */
    hyper disk_allocated;      /* Version 1.0: not filled */

    longstring *disk_backing_file;   
    /* For .cow disks this component indicates the name of the backing file */
    
    int disk_use_count;
    /* For .dsk disks this component says how many .cow files refer to it.
     * The number is only exact for .dsk disks in /disks, not for disks in
     * /shared/disks.
     */

    bool disk_rw;
    /* Whether the disk is read/write. If not, the disk is at least read-only,
     * because inaccessible disks are usually omitted from the list of disks.
     * This attribute refers only to the access privileges. If a file is
     * effectively read-only because of a read lock, this is not reported
     * here.
     */

    lock_info disk_lock;
    /* The lock status. As of UMLMON 1.0.1, only files in /shared can be locked.
     */

    longstring disk_extension<>;
    /* Tokens describing extension information.
     * Currently defined extensions:
     * MAJOR  MINOR   EXTENSION    COMMENT
     * -------------------------------------------------------------------------------
     * 1      0       -            no extension defined
     */
};
/* Report about an existing disk */


union r_diskinfo switch (disk_status ds) {
 case DISK_EXISTS:
     disk_info disk_info;
 case DISK_BEING_CREATED:
     void;

 case DISK_ERROR:
     longstring disk_error_message;

     /* DISK_ERROR is a temporary status. It is only returned once after 
      * the failed modification of the disk.
      */
};
/* Part of the return value of the disk_files procedure. */


struct r_disk {
    longstring disk_name;           /* The file name of the disk */
    bool       disk_is_cow;         /* Whether this is a .cow disk */
    bool       disk_owned;          /* Whether this disk is in /disks */
    r_diskinfo disk_info;           /* Status and information */
};
/* Part of the return value of the disk_files procedure. */


typedef r_disk r_disks<>;
/* An array of disks */    

typedef longstring options<>;
/* Options, see the procedures for explanations */

struct r_disk_options {
    longstring disk_opts_create<>;
    longstring disk_opts_create_cow<>;
    longstring disk_opts_delete<>;
    longstring disk_opts_copy<>;
    longstring disk_opts_resize<>;
    /* For every disk procedure the corresponding array lists the
     * options that can be passed to the server and that are
     * actually interpreted.
     */
};



struct config_param {
    longstring param_name;
    /* The name (as in the config file). The name must fit to the parameter type */

    longstring param_value;
    /* The value as string. The syntax is the same as in the config file. */
};
/* Describes a single configuration parameter */


typedef config_param config_params<>;
/* Just multiple parameters */


struct config_param_set {
    /* Configuration parameters. In cfg_<type> the parameter or list of parameters
     * is stored. In cfg_<type>_rw it is indicated whether the parameter can be
     * changed at all using the config_set. In cfg_<type>_hot it is indicated
     * whether devices can be added or removed while the virtual machine is
     * running.
     */

    config_param cfg_privsetup<>;
	/* Parameters that can only be changed by the administrator:
	 *   jaildir, logdir, vmuser, vmgroup.
	 * This array is empty for calls over TCP (security reasons).
	 */

    config_param cfg_start;
    bool         cfg_start_rw;         /* true */

    config_param *cfg_logfile;
    bool         cfg_logfile_rw;       /* true */

    config_param cfg_kernel;
    bool         cfg_kernel_rw;	       /* true */

    config_param cfg_kernelargs<>;
    bool         cfg_kernelargs_rw;    /* true */

    config_param cfg_mem;
    bool         cfg_mem_rw;           /* true */
    bool         cfg_mem_hot;          /* false */

    config_param cfg_con<>;
    bool         cfg_con_rw;           /* true */
    bool         cfg_con_hot;          /* false */

    config_param cfg_ssl<>;
    bool         cfg_ssl_rw;           /* true */
    bool         cfg_ssl_hot;          /* false */

    config_param cfg_ubd<>;
    bool         cfg_ubd_rw;           /* true */
    bool         cfg_ubd_hot;          /* false */

    config_param cfg_eth<>;
    bool         cfg_eth_rw;           /* false */
    bool         cfg_eth_hot;          /* false */

    config_param cfg_extension<>;
    /* These are extension parameters, to be used in future extension of the
     * Version 1 RPC protocol. They are usually read/write, as read-only
     * parameters can usually been accessed by cfg_privsetup. It is undefined
     * whether extensions can be added/enabled or removed/disabled while
     * the machine is running.
     *
     * Currently defined extensions:
     * MAJOR  MINOR   EXTENSION    COMMENT
     * -------------------------------------------------------------------------------
     * 1      0       -            no extension defined
     * 1      1       sync_disks   Lists the ubd disks to open in sync mode.
     *                             Keyword "all" forces all disks to be
     *                             synchronous (the default).
     */

};
/* The configuration parameters split by type */


struct kernel {
    longstring kernel_file;       /* The file to use in the kernel config option */
    longstring kernel_name;       /* A name or comment */
};

typedef kernel kernels<>;
/* A list of kernels */



typedef longstring credentials<2>;
/* An array of credentials. Currently, one can only pass two credentials at once
 * (i.e. the master password and the machine password).
 */


program Ctrl {
    version V1 {
	void ping(void) = 0;

	/* --- 0xx: Connection control --- */

	void ensure_id(longstring) = 1;
	/* Checks whether the monitor has the passed ID. If so, nothing happens, 
	 * and further calls can be made. If the monitor does not have this ID
         * the connection will be immediately closed.
	 *
	 * It is not necessary to be authenticated.
	 *
	 * This procedure is also used by the redirection service: To connect to a
	 * certain monitor, connect with the umlredir port, and call ensure_id with the
	 * desired ID.
	 */

	int minor_protocol_version(void) = 2;
	/* Returns the minor protocol version spoken by the server */
	
	options available_extensions(void) = 3;
	/* The server can announce certain extensions with this procedure.
         *
         * MAJOR  MINOR   EXTENSION    COMMENT
         * -------------------------------------------------------------------------------
         * 1      0       -            Nothing is announced.
         * 1      1       sync_disks   The sync_disks config parameter is
         *                             understood.
	 */
	
        /* 4-99: Area for possible extensions */

	/* --- 1XX: Status queries --- */

	longstring get_id(void) = 100;
	/* Return the name of the machine */

	uml_status get_status(void) = 101;
	/* Return the status of the machine (as seen by the monitor) */

	uml_mon_terms get_monitored_terminals(void) = 102;
	/* Return the array of all monitored terminals (serial lines and consoles).
	 * The array reflects the real monitoring state, not the configured state.
	 */

	longstring_opt get_ssl(int) = 103;
	/* Returns the definition of the serial line number N, or null if this line
	 * is unknown. This procedure queries the machine directly using the
         * mconsole socket.
	 */

	longstring_opt get_con(int) = 104;
	/* Returns the definition of the console number N, or null if this console
	 * is unknown. This procedure queries the machine directly using the
         * mconsole socket.
	 */

	longstring_opt get_ubd(int) = 105;
	/* Returns the definition of the disk ubd N, or NULL if this disk is unknown.
	 * This procedure queries the machine directly using the
         * mconsole socket.
	 */

	longstring_opt get_eth(int) = 106;
	/* Not implemented */

	int_opt port(void) = 107;
	/* Returns the TCP port number on which this monitor listens. It is
	 * unspecified on which interface the monitor listens.
	 */

	r_logfile_read logfile_read(longstring, int, int) = 108;
	/* Reads one of the log files. The first argument selects the file, it is
	 * either the name of a terminal ("con0"), or the string "monitor" for the
	 * main log file. The second argument is the first line to read, negative
	 * number count from the end (e.g. -1 is the last line). The third argument,
	 * which must be positive, are the number of lines to read. This is limited
	 * to 1000.
	 */

	kernels announced_kernels(void) = 109;
	/* Returns the array of announced kernels that can be started */
         

        /* 109-199: Area for possible extensions */

	/* --- 2XX: Status changes --- */

	r_simple start(arg_list) = 200;
	/* Start the machine. The argument is a list of further kernel parameters
	 * to add to the already existing parameters.
	 *
	 * - CODE_OK: The UML process is already running
	 * - CODE_IN_PROGRESS: The creation of the UML process has been triggered
	 * - CODE_ERROR: an error occurred.
	 * - CODE_INTERRUPED: not possible
	 */

	r_simple stop(void) = 201;
	/* Suspends the machine. Returns CODE_OK or CODE_ERROR */

	r_simple continue(void) = 202;
	/* Continues the machine. Returns CODE_OK or CODE_ERROR */

	r_simple halt(void) = 203;
	/* Sends a HALT request to the machine. Returns CODE_OK or CODE_ERROR */

	r_simple ctrlaltdel(void) = 204;
	/* Sends CTRL-ALT-DEL to the machine. Returns CODE_OK or CODE_ERROR */

	r_simple kill(void) = 205;
	/* Kills the UML process. Returns CODE_OK or CODE_ERROR */

        /* 206-299: Area for possible extensions */

	/* --- 3XX: Operational stuff --- */

	r_simple rotate(void) = 300;
	/* Reopens all log files. Should be called after log file rotation.
	 * Returns CODE_OK or CODE_ERROR
	 */

	r_simple sysrq(sysrq_arg) = 301;
	/* Sends a SYSRQ request to the machine. Returns CODE_OK or CODE_ERROR */

        /* 302-399: Area for possible extensions */

	/* --- 4XX: Communication --- */

	r_connect connect (longstring) = 400;
	/* This procedure connects with a terminal (console or serial line).
	 * Once connected, one can read from the terminal using the "read"
	 * procedure and write to the terminal using the "write" procedure.
	 * It is possible (and reasonable) to read and write at the same time
	 * using asynchronous RPC calls. The terminal remains in connected state
	 * until it is disconnected using the "disconnect" procedure, or the
	 * connection times out. The latter happens pretty quickly. It is recommended
	 * to call "read" or "write" several times per minute to avoid timeouts.
	 *
	 * The argument is the name of the console or serial line.
	 *
	 * Returns CODE_OK or CODE_ERROR. The CODE_OK contains the [term_handle]
         * that must be passed to subsequent invocations of [read] and [write].
         */

	r_read read (term_handle, int) = 401;
	/* Reads data from a connected terminal, i.e. data that the user process
	 * has previously output to the terminal.
	 *
	 * The first argument identifies the terminal to read from. The second
	 * argument is the timeout in seconds. The procedure blocks until 
	 * there is data to read or the timeout is exceeded or the terminal
	 * has been disconnected. 
	 *
         * - CODE_OK: data have been read
         * - CODE_ERROR: cannot read because of error
         * - CODE_INTERRUPT: timeout or disconnect
         * - CODE_IN_PROGRESS: not possible
	 */

	r_write write(term_handle, longstring, int) = 402;
	/* Writes data to a connected terminal, i.e. data that will be seen as
	 * input by user processes reading from the terminal.
	 *
	 * The first argument identifies the terminal to read from. The second
	 * argument are the data to write. The third argument is the timeout in
	 * seconds. The procedure blocks until the data could be written or the
	 * timeout is exceeded or the terminal has been disconnected. 
	 *
         * - CODE_OK: data have been completely written
         * - CODE_ERROR: cannot write because of error
         * - CODE_INTERRUPT: timeout or disconnect. In wr_length it is returned how
         *   much bytes could be written.
         * - CODE_IN_PROGRESS: not possible
	 */

	r_disconnect disconnect(term_handle, bool) = 403;
	/* Disconnects the terminal identified by the first argument.
	 *
	 * If a true value is passed as second argument, a forced disconnect happens.
	 * In this case, only the ct_name component of the first argument is used
	 * to identify the terminal.
	 *
	 * Returns CODE_OK or CODE_ERROR.
	 */

	bool is_connected(longstring) = 404;
	/* Returns whether the terminal (name in arg) is connected */


	/* --- 42X: Terminal emulator --- */

	/* The terminal emulator implements most functions of the Linux console.
	 * It is optional to use the terminal emulator. If doing so, one should
	 * no longer call "read" and "write" to get and send terminal data
	 * directly as this bypasses the emulator.
	 *
	 * The emulator does not automatically receive data of the terminal.
	 * One must explicitly call te_read to pass terminal data to the
	 * emulator.
	 */

	r_simple te_init (term_handle, int, int) = 420;
	/* Initializes the terminal emulator for a connected terminal. 
	 * The two ints are the width and the height. Limit: width * height <= 20000.
	 *
	 * Note: For large screens a lot of memory is consumed!
	 */
	
	r_simple te_read(term_handle, int) = 421;
	/* Same as [read], but the read data are passed to the emulator instead of
	 * being returned to the caller.
	 */
	
	r_simple te_write_keys(term_handle, keys, int) = 422;
	/* Emulates keypresses in the terminal emulator. The first argument identifies
	 * the terminal. The second argument is a list of keys to emulate one after
	 * the other. The third argument is a timeout value.
	 *
	 * On timeout, CODE_INTERRUPT is returned, but it is not indicated how many
	 * keys could be processed.
	 */
	
	r_te_state_opt te_state(term_handle) = 423;
	/* Returns the state of the emulator (screen contents, cursor position, etc.) */

	r_simple te_reset(term_handle) = 424;
	/* Performs a terminal reset */

        /* 425-499: Area for possible extensions */

	/* --- 5XX: Disk interface --- */

	/* Files in <jail>/disks and <jail>/shared/disks are recognized as disks:
	 * - <filename>.dsk is a simple disk
	 * - <filename>.cow is a copy-on-write disk
         * - <filename>.dev is a block device
	 *
         * There are limitations for the shared disks. Although VMs can access
         * them read/write, it is currently not possible to modify these files
         * using this RPC interface.
	 *
	 * Disk manipulation is only permitted when the machine is inactive.
	 */

	r_disks disk_files(void) = 500;
	/* Get the statuses of all disk files */

	r_simple disk_create(longstring,int,options) = 501;
	/* Create a disk. The argument 1 must match "/disks/*.dsk". arg2 is the
	 * size in Megabytes. arg3 are the options:
         *
         * MAJOR  MINOR   OPTION       COMMENT
         * -------------------------------------------------------------------------------
         * 1      0       sparse       The file is created sparse
	 *
	 * The disk is usually created in the background. You can query the current status
	 * by calling [disk_files].
	 *
	 * - CODE_OK: disk has been created
	 * - CODE_IN_PROGRESS:  The disk is created in the background. You can query
	 *   the current status by calling [disk_files].
	 * - CODE_ERROR: something went wrong.
	 * - CODE_INTERRUPTED: not possible.
	 */

	r_simple disk_create_cow(longstring,longstring,options) = 502;
	/* Create a cow file. The first argument is the backing disk, which must be
	 * an existing "/disks/*.dsk" file or "/shared/disks/*.dsk" file.
	 * The 2nd arg is the name of the cow file: "/disks/*.cow".
         * arg3 are the options:
         *
         * MAJOR  MINOR   OPTION       COMMENT
         * -------------------------------------------------------------------------------
         * 1      0       -            Currently no options defined.
	 *
	 * Return codes are as for disk_create.
	 */

	r_simple disk_delete(longstring,options) = 503;
	/* Deletes a disk file or cow file. It is not possible to delete disk files
	 * to which a cow file refers.
         * arg3 are the options:
         *
         * MAJOR  MINOR   OPTION       COMMENT
         * -------------------------------------------------------------------------------
         * 1      0       -            Currently no options defined.
	 *
	 * - CODE_OK: disk has been deleted
	 * - CODE_ERROR: something went wrong.
	 * - CODE_IN_PROGRESS, CODE_INTERRUPTED: not possible.
	 */

	r_simple disk_copy(longstring,longstring,options) = 504;
	/* Copies a disk file. Arg1 is the source file, arg2 is the destination file.
	 *
	 * The copy is usually done in the background. The destination file (which must
	 * not exist before the copy) is set to DISK_BEING_CREATED during the copy.
	 * Return codes as for disk_create.
         *
         * arg3 are the options:
         *
         * MAJOR  MINOR   OPTION       COMMENT
         * -------------------------------------------------------------------------------
         * 1      0       -            Currently no options defined.
	 */

	r_simple disk_resize(longstring,int,options) = 505;
	/* Resize a disk. This is only possible for .dsk files in /disks. Arg1 is the
	 * file name, arg2 is the new size in megs. arg3 are the options:
         *
         * MAJOR  MINOR   OPTION       COMMENT
         * -------------------------------------------------------------------------------
         * 1      0       sparse       If the file grows, the new parts are created
         *                             sparse
	 *
	 *
	 * The resize operation is usually done in the background. The disk status is
	 * set to DISK_BEING_CREATED during the operation.
	 * Return codes as for disk_create.
	 */
	
	r_disk_options disk_options(void) = 506;
	/* Returns the defined options for the above disk procedures */

        /* 507-599: Area for possible extensions */

	/* --- 6XX: Configuration interface --- */

	/* One can only change certain configuration parameters. The changed set
	 * of parameters is called the dynamic configuration.
	 *
	 * The configuration becomes active the next time the machine is started.
	 */

	config_param_set config_get(void) = 600;
	/* Returns the current configuration */

	r_simple config_set(config_param_set) = 601;
	/* Sets the dynamic configuration parameters. These parameters are 
         * evaulated the next time the machine is started.
	 */

	r_simple config_hot_add(config_params) = 602;
	/* Adds new devices while the machine is running.
         * - This is currently not implemented and always returns CODE_ERROR.
         */

	r_simple config_hot_remove(config_params) = 603;
	/* Removes devices while the machine is running.
         * - This is currently not implemented and always returns CODE_ERROR.
         */

        /* 602-699: Area for possible extensions */

	/* --- 7XX: Authentication --- */

	/* Authentication is only enforced for the TCP interface! However, it is
	 * allowed to call the authentication procedures for the Unix Domain connections,
	 * too.
	 *
	 * In general, authentication is done by calling first auth_get_challenge
	 * and then auth_pass_credentials. auth_get_challenge returns a random
	 * number, and the client convinces the server that the password is known
	 * by changing the random number using an algorithm where the password is
	 * a parameter. The new number is called a "credential", and passed to the
	 * server using auth_pass_credentials.
	 */

	longstring auth_get_challenge(void) = 700;
	/* Returns the challenge string. This procedure can only be called once
	 * per connection. (Further calls will return new challenges, but it is
	 * not possible to use them for authentication purposes.)
	 */

	bool auth_pass_credentials(longstring,credentials) = 701;
	/* Passes the credentials for the previously retrieved challenge in arg2.
         * It is allowed to pass two credentials simultaneously. Authentication
         * succeeds if one of the credentials is accepted.
	 *
	 * arg1 selects the HMAC algorithm (see below).
	 *
	 * Returns true when authentication is successful, else false.
	 *
	 * This procedure can only be called once per connection.
	 *
	 * The credentials are computed from the challenge by:
	 *
	 * credential = ALGO( challenge , MD5(password) )
	 *
	 * Algorithms:
	 * - "HMAC-SHA1": The HMAC-SHA1 algorithm as defined in RFC 2104.
	 */

	r_simple auth_change_password(longstring) = 702;
	/* Changes the machine password. The new password is encrypted using the
	 * old password as key, such that no password is passed in clear text.
	 *
	 * Note that this procedure cannot check whether the client knows the
	 * old password. This should be ensured before calling this procedure!
	 * If the client uses the wrong password, the password is changed 
	 * nevertheless, but nobody knows it.
	 *
	 * The argument is:
	 *
	 * arg = AES-CBC( MD5(old_password), MD5(new_password) )
         *
         * i.e. the hash of the old password is used as 128 bit key to encrypt
	 * the hash of the new password using the AES algorithm in CBC mode.
	 */

	void auth_reset(void) = 703;
	/* Resets the authentication state of the connection. This works even for
	 * Unix domain connections which start normally authenticated.
	 */

        /* 704-799: Area for possible extensions */

	/* --- 8XX: Miscellenous --- */

	r_site site_command(arg_list) = 800;
	/* Start the site command, and pass the argument list to it. 
	 * Return codes:
	 * - CODE_OK: exit code 0. The site_result is the collected output to
	 *   stdout.
	 * - CODE_ERROR: exit code > 0, or other error. The error_message may
	 *   contain the output to stderr.
	 */

        /* 800-899: Area for possible extensions */

    } = 1;  /* The major version number of the protocol */
} = 0;   /* Not registered in portmapper so number does not matter */


program Internal {
    /* Internally used functions */

    version V1 {
	void ping(void) = 0;
	
	void uml_statchange_running (int) = 1;
	/* The UML process has just started. The argument is the PID */

	void uml_statchange_exit (int) = 2;
	/* The UML process has just exited. The argument is the exit code */

	void uml_statchange_error (void) = 3;
	/* The UML process has just exited because of a signal or error */

    } = 1;

} = 1;   /* Not registered in portmapper so number does not matter */


struct monitor {
    longstring monitor_id;    /* The ID of the monitor */
    int_opt    monitor_port;  /* The TCP port the monitor listens on */
};
/* Describes a monitor by its monitor_id and the optional TCP port */


typedef monitor monitors<>;
/* An array of monitor descriptions */


program Directory {
    /* The Directory interface is implemented by the umldir program. It is
     * listening locally on /var/lib/umlmon/directory/directory, or on the TCP
     * port "umldir" (as defined in /etc/services).
     */

    version V1 {
	void ping(void) = 0;

	monitors get_monitors(void) = 1;
	/* Returns an array of all locally running monitors */

	int_opt  get_port(longstring) = 2;
	/* Returns the Internet port of a monitor instance */

	bool     have_redirection(void) = 3;
	/* Whether the redirection service is enabled. If so, an alternate method of 
	 * connecting to a monitor is defined as follows:
	 * - Connect to the umlredir port 
	 * - Call ensure_id to select the monitor, wait for the reply
	 * - The port is now redirected to the monitor. Go on as usual: Authenticate,
	 *   and call further methods.
	 */

    } = 1;

} = 2;   /* Not registered in portmapper so number does not matter */
